#!/usr/local/bin/ipython3
import astropy,os,scipy,scipy.stats,math
import matplotlib.pyplot as plt
import numpy as np
from astropy.io import fits
from scipy.ndimage import gaussian_filter1d

def bias_sim(size=(4096,4096),median=1000,stdev=5.):
    print(size,median,stdev)
#    data=[size[0],size[1]]
    data=np.trunc(np.random.randn(size[0],size[1])*stdev+median)
#    data=np.random.randn(size[0],size[1])*stdev+median
    return data

def sky_sim(size=(4096,4096),gain=1.,zeropoint=25.,sky_mag=22.,pixel_size=0.454,exptime=60.):
    sky=10**((sky_mag-zeropoint)/(-2.5))*pixel_size**2*exptime/gain
    print(sky,sky_mag,zeropoint,exptime,pixel_size)
    data=np.trunc(np.random.randn(size[0],size[1])*np.sqrt(sky)+sky)
    return data

def bad_sim(template,size=(4096,4096)):
#    print(size,median,stdev)
#    data=[size[0],size[1]]
    data=np.random.choice(template,size)
    return data

def pattern(x,p):
    a,b,c=p
    return np.trunc(a*np.sin((x+b)*np.pi/c))

def trail_sim(size=(4096,4096),position_start=(0,0),velocity=(1.,1.),mag=15.,exptime=60,pixel_size=1.3,fwhm=3.0,gain=1,zeropoint=25.):
    data=np.zeros(size)
    print(position_start,velocity,mag)
    for i in range(exptime):
        position_now=position_start+i*velocity
#        x_now,y_now=x_start+i*velocity_x,y_start+i*velocity_y
        mx,my=np.int(position_now[0]),np.int(position_now[1])
        x,y=np.meshgrid(np.linspace(mx-25,mx+25,51),np.linspace(my-25,my+25,51))
        d=np.sqrt((x-position_now[0])**2+(y-position_now[1])**2)
        sigma=fwhm/2.35482/pixel_size
        g=np.exp(-((d)**2/(2.0*sigma**2)))#*flux0
        flux0=10**((mag-zeropoint)/(-2.5))/gain
        s=np.random.poisson(g/np.sum(g)*flux0)        
        data[mx-26:mx+25,my-26:my+25]+=s
    return data

#s=np.random.standard_exponential(1000)
#plt.hist(s,bins=10)
#plt.show()
pixel_size,exptime,gain,zeropoint=1.3,60.,1.,23.0
x1=np.linspace(0,9099,9100)

bias=bias_sim((1100,1100),median=1000,stdev=5.)
data=np.copy(bias)

sky=sky_sim((1100,1100),zeropoint=23.,sky_mag=22.2,exptime=60.,pixel_size=1.3)
data+=sky

sin_p=[np.random.rand(1100)*100.-50.,np.random.rand(1100)*2000.-1000.,np.random.rand(1100)*1000.]

#for i in range(9100):
#    data[i]+=pattern(x1,[sin_p[0][i],sin_p[1][i],sin_p[2][i]])

position=np.random.rand(2,10000)*1000.+30.
fwhm=np.random.rand(1000)*4.0
flux=np.random.standard_exponential(10000)*(-2)+22.

for i in range(10):
    velo=(np.random.rand(2)-0.5)*(-6.)
    posi=100.+np.random.rand(2)*800.
    trail=trail_sim(size=(1100,1100),velocity=velo,position_start=posi,zeropoint=23.0,mag=np.random.standard_exponential(1)*(-1.)+17.,exptime=60)
    data+=trail
#print(position,fwhm)

for i in range(10000):
    xc,yc=position[0,i],position[1,i]
    mx,my=np.int(xc),np.int(yc)

    x,y=np.meshgrid(np.linspace(mx-25,mx+25,51),np.linspace(my-25,my+25,51))
    d=np.sqrt((x-xc)**2+(y-yc)**2)
    sigma=2.5/2.35482/pixel_size
    g=np.exp(-((d)**2/(2.0*sigma**2)))#*flux0
    flux0=10**((flux[i]-zeropoint)/(-2.5))*exptime/gain
#    print(i,flux0,flux[i])
#    flux0=65535*50.
    s=np.random.poisson(g/np.sum(g)*flux0)
#    print(np.sum(s),flux[i],xc,yc,sigma)
    mask=(s>=65535)
#    s0=1*mask*65535+(1-mask)*s
    s1=np.copy(s)
    for n in range(51):
        if(np.sum(mask[:,n])>0):
#            print(n,np.sum(mask[:,n]))
            s1[:,n]=scipy.ndimage.filters.gaussian_filter1d(s[:,n],sigma=30.)
#                print(np.mean(s[min(0,m-1):max(m+1,31),min(0,n-1):max(n+1,31)]))
#                s1[m,n]=np.trunc(np.mean(s[max(0,m-1):min(m+1,51),max(0,n-1):min(n+1,51)]))
    mask=(s1>=65535)
    s0=1*mask*65535+(1-mask)*s1

    data[mx-26:mx+25,my-26:my+25]+=s0

#bad=fits.open('bb_1.fit')
#data1=bad[0].data.flatten()

#data2=np.random.choice(data1,size=(4032,4096))
#data1_mask=(data1<=0.95)
#data2_mask=(data2<=0.95)
#print(np.sum(data1_mask),np.sum(data2_mask))
#data=bad[1].data[:2048,:2016]
#data1=data.flatten()
#c,low,upp=scipy.stats.sigmaclip(data1,3.0,3.0)
#data2=np.trunc(np.random.randn(2048,2016)*np.std(c)+np.median(c))
#data3=data2.flatten()
#plt.axis([0.9*np.median(c),1.1*np.median(c),0.,10000.])
#print(np.shape(data),np.median(c),np.std(c))
#print(np.shape(data2),np.median(data2),np.std(data2))
#plt.hist(data1,bins=200,range=[0.95*np.median(c),1.05*np.median(c)])
#print(np.shape(c)[0]/4032/4096)
#plt.hist(data3,bins=200,range=[0.95*np.median(c),1.05*np.median(c)])
#        plt.plot(x,cc,'r',lw=3)
#        plt.plot(sky[j,:],sky[j,:]/sky[id_s,:],'o')
#        plt.xlabel('Sky ADU') 
#        plt.ylabel('Sky ADU / Sky ADU (standard)')
#plt.show()
#hdu=fits.PrimaryHDU(data2_mask*1)
hdu=fits.PrimaryHDU(data)
hdu.writeto('test.fits',clobber='True')
#        raw_input()
